/*
 *  readdistcolumnfilecommand.cpp
 *  Dotur
 *
 *  Created by Sarah Westcott on 1/2/09.
 *  Copyright 2009 Schloss Lab UMASS Amherst. All rights reserved.
 *
 */

#include "readdistcolumnfilecommand.h"

ReadDistColumnFileCommand::ReadDistColumnFileCommand(){
	try {
		globaldata = GlobalData::getInstance();
		
		filename = globaldata->inputFileName;
		format = globaldata->getFormat();	
		
		read = new ReadColumnMatrix(filename);	
			
		if(globaldata->getPrecision() != ""){
			convert(globaldata->getPrecision(), precision);	
		}
		
		if(globaldata->getCutOff() != ""){
			convert(globaldata->getCutOff(), cutoff);	
			cutoff += (5 / (precision * 10.0));
		}
		read->setCutoff(cutoff);
	
		nameMap = new NameAssignment(globaldata->getNameFile());
		nameMap->readMap(1,2);
			
	}
	catch(exception& e) {
		cout << "Standard Error: " << e.what() << " has occurred in the ReadDistColumnFileCommand class Function ReadDistColumnFileCommand. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
	catch(...) {
		cout << "An unknown error has occurred in the ReadDistColumnFileCommand class function ReadDistColumnFileCommand. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
}

//**********************************************************************************************************************
ReadDistColumnFileCommand::~ReadDistColumnFileCommand(){
	delete read;
	delete nameMap;
}

//**********************************************************************************************************************
int ReadDistColumnFileCommand::execute(){
	try {
		read->read(nameMap);
		globaldata->setListVector(read->getListVector());
		globaldata->setSparseMatrix(read->getMatrix());
		return 0;
	}
	catch(exception& e) {
		cout << "Standard Error: " << e.what() << " has occurred in the ReadDistColumnFileCommand class Function execute. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
	catch(...) {
		cout << "An unknown error has occurred in the ReadDistColumnFileCommand class function execute. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
}
