/*
 *  readlistcommand.cpp
 *  Mothur
 *
 *  Created by Sarah Westcott on 1/20/09.
 *  Copyright 2009 Schloss Lab UMASS Amherst. All rights reserved.
 *
 */

#include "readlistcommand.h"

//**********************************************************************************************************************
ReadListFileCommand::ReadListFileCommand(){
	try {
		globaldata = GlobalData::getInstance();
		filename = globaldata->inputFileName;	
		read = new ReadPhilFile(filename);
		//read in group map info.
		groupMap = new GroupMap(globaldata->getGroupFile());
	}
	catch(exception& e) {
		cout << "Standard Error: " << e.what() << " has occurred in the ReadListFileCommand class Function ReadListFileCommand. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
	catch(...) {
		cout << "An unknown error has occurred in the ReadListFileCommand class function ReadListFileCommand. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
}

//**********************************************************************************************************************

ReadListFileCommand::~ReadListFileCommand(){
	delete read;
	//delete groupMap;
}

//**********************************************************************************************************************

int ReadListFileCommand::execute(){
	try {
		read->read(&*globaldata); 
		groupMap->readMap();
		globaldata->gGroupmap = groupMap;
		
		shared = new SharedCommand();
		shared->execute();
		parselist = new ParseListCommand();
		parselist->execute();
		
		
		return 0;
	}
	catch(exception& e) {
		cout << "Standard Error: " << e.what() << " has occurred in the ReadListFileCommand class Function execute. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
	catch(...) {
		cout << "An unknown error has occurred in the ReadListFileCommand class function execute. Please contact Pat Schloss at pschloss@microbio.umass.edu." << "\n";
		exit(1);
	}
}

//**********************************************************************************************************************
